@@ -15,21 +15,24 @@
 
 import re
 
-from neutron.api.v2 import attributes
+import netaddr
 from neutron.callbacks import events
 from neutron.callbacks import registry
 from neutron.callbacks import resources
+from neutron.common import ipv6_utils
+from neutron.db import api as db_api
 from neutron.db import common_db_mixin as base_db
-from neutron import manager
 from neutron.plugins.common import constants
-from neutron_lib import constants as n_constants
+from neutron_lib import constants as n_const
 from neutron_lib import exceptions as n_exc
+from neutron_lib.plugins import directory
 from oslo_db import exception
 from oslo_log import log as logging
 from oslo_utils import excutils
 from oslo_utils import uuidutils
 from sqlalchemy import orm
 from sqlalchemy.orm import exc
+from sqlalchemy.orm import lazyload
 
 from neutron_lbaas._i18n import _
 from neutron_lbaas import agent_scheduler
@@ -54,13 +57,18 @@ class LoadBalancerPluginDbv2(base_db.CommonDbMixin,
 
     @property
     def _core_plugin(self):
-        return manager.NeutronManager.get_plugin()
+        return directory.get_plugin()
 
     def _get_resource(self, context, model, id, for_update=False):
         resource = None
         try:
             if for_update:
-                query = self._model_query(context, model).filter(
+                # To lock the instance for update, return a single
+                # instance, instead of an instance with LEFT OUTER
+                # JOINs that do not work in PostgreSQL
+                query = self._model_query(context, model).options(
+                    lazyload('*')
+                ).filter(
                     model.id == id).with_lockmode('update')
                 resource = query.one()
             else:
@@ -89,33 +97,68 @@ class LoadBalancerPluginDbv2(base_db.CommonDbMixin,
                                            filters=filters)
         return [model_instance for model_instance in query]
 
-    def _create_port_for_load_balancer(self, context, lb_db, ip_address):
-        # resolve subnet and create port
-        subnet = self._core_plugin.get_subnet(context, lb_db.vip_subnet_id)
-        fixed_ip = {'subnet_id': subnet['id']}
-        if ip_address and ip_address != attributes.ATTR_NOT_SPECIFIED:
-            fixed_ip['ip_address'] = ip_address
+    def _create_port_choose_fixed_ip(self, fixed_ips):
+        # Neutron will try to allocate IPv4, IPv6, and IPv6 EUI-64 addresses.
+        # We're most interested in the IPv4 address. An IPv4 vip can be
+        # routable from IPv6. Creating a port by network can be used to manage
+        # the dwindling, fragmented IPv4 address space. IPv6 has enough
+        # addresses that a single subnet can always be created that's big
+        # enough to allocate all vips.
+        for fixed_ip in fixed_ips:
+            ip_address = fixed_ip['ip_address']
+            ip = netaddr.IPAddress(ip_address)
+            if ip.version == 4:
+                return fixed_ip
+        # An EUI-64 address isn't useful as a vip
+        for fixed_ip in fixed_ips:
+            ip_address = fixed_ip['ip_address']
+            ip = netaddr.IPAddress(ip_address)
+            if ip.version == 6 and not ipv6_utils.is_eui64_address(ip_address):
+                return fixed_ip
+        for fixed_ip in fixed_ips:
+            return fixed_ip
+
+    def _create_port_for_load_balancer(self, context, lb_db, ip_address,
+                                       network_id=None):
+        if lb_db.vip_subnet_id:
+            assign_subnet = False
+            # resolve subnet and create port
+            subnet = self._core_plugin.get_subnet(context, lb_db.vip_subnet_id)
+            network_id = subnet['network_id']
+            fixed_ip = {'subnet_id': subnet['id']}
+            if ip_address and ip_address != n_const.ATTR_NOT_SPECIFIED:
+                fixed_ip['ip_address'] = ip_address
+            fixed_ips = [fixed_ip]
+        elif network_id and network_id != n_const.ATTR_NOT_SPECIFIED:
+            assign_subnet = True
+            fixed_ips = n_const.ATTR_NOT_SPECIFIED
+        else:
+            attrs = _("vip_subnet_id or vip_network_id")
+            raise loadbalancerv2.RequiredAttributeNotSpecified(attr_name=attrs)
 
         port_data = {
             'tenant_id': lb_db.tenant_id,
             'name': 'loadbalancer-' + lb_db.id,
-            'network_id': subnet['network_id'],
-            'mac_address': attributes.ATTR_NOT_SPECIFIED,
+            'network_id': network_id,
+            'mac_address': n_const.ATTR_NOT_SPECIFIED,
             'admin_state_up': False,
             'device_id': lb_db.id,
-            'device_owner': n_constants.DEVICE_OWNER_LOADBALANCERV2,
-            'fixed_ips': [fixed_ip]
+            'device_owner': n_const.DEVICE_OWNER_LOADBALANCERV2,
+            'fixed_ips': fixed_ips
         }
 
         port = self._core_plugin.create_port(context, {'port': port_data})
         lb_db.vip_port_id = port['id']
-        for fixed_ip in port['fixed_ips']:
-            if fixed_ip['subnet_id'] == lb_db.vip_subnet_id:
-                lb_db.vip_address = fixed_ip['ip_address']
-                break
 
-        # explicitly sync session with db
-        context.session.flush()
+        if assign_subnet:
+            fixed_ip = self._create_port_choose_fixed_ip(port['fixed_ips'])
+            lb_db.vip_address = fixed_ip['ip_address']
+            lb_db.vip_subnet_id = fixed_ip['subnet_id']
+        else:
+            for fixed_ip in port['fixed_ips']:
+                if fixed_ip['subnet_id'] == lb_db.vip_subnet_id:
+                    lb_db.vip_address = fixed_ip['ip_address']
+                    break
 
     def _create_loadbalancer_stats(self, context, loadbalancer_id, data=None):
         # This is internal method to add load balancer statistics.  It won't
@@ -147,6 +190,12 @@ class LoadBalancerPluginDbv2(base_db.CommonDbMixin,
     def _load_id(self, context, model_dict):
         model_dict['id'] = uuidutils.generate_uuid()
 
+    def check_subnet_exists(self, context, subnet_id):
+        try:
+            self._core_plugin.get_subnet(context, subnet_id)
+        except n_exc.SubnetNotFound:
+            raise loadbalancerv2.EntityNotFound(name="Subnet", id=subnet_id)
+
     def assert_modification_allowed(self, obj):
         status = getattr(obj, 'provisioning_status', None)
         if status in [constants.PENDING_DELETE, constants.PENDING_UPDATE,
@@ -204,30 +253,91 @@ class LoadBalancerPluginDbv2(base_db.CommonDbMixin,
                     model_db.operating_status != operating_status):
                 model_db.operating_status = operating_status
 
-    def create_loadbalancer(self, context, loadbalancer, allocate_vip=True):
+    def create_loadbalancer_graph(self, context, loadbalancer,
+                                  allocate_vip=True):
+        l7policies_ids = []
         with context.session.begin(subtransactions=True):
-            self._load_id(context, loadbalancer)
-            vip_address = loadbalancer.pop('vip_address')
-            loadbalancer['provisioning_status'] = constants.PENDING_CREATE
-            loadbalancer['operating_status'] = lb_const.OFFLINE
-            lb_db = models.LoadBalancer(**loadbalancer)
-            context.session.add(lb_db)
-            context.session.flush()
-            lb_db.stats = self._create_loadbalancer_stats(
-                context, lb_db.id)
-            context.session.add(lb_db)
+            listeners = loadbalancer.pop('listeners', [])
+            lb_db = self.create_loadbalancer(context, loadbalancer,
+                                             allocate_vip=allocate_vip)
+            for listener in listeners:
+                listener['loadbalancer_id'] = lb_db.id
+                default_pool = listener.pop('default_pool', None)
+                if (default_pool and
+                        default_pool != n_const.ATTR_NOT_SPECIFIED):
+                    default_pool['loadbalancer_id'] = lb_db.id
+                    hm = default_pool.pop('healthmonitor', None)
+                    if hm and hm != n_const.ATTR_NOT_SPECIFIED:
+                        hm_db = self.create_healthmonitor(context, hm)
+                        default_pool['healthmonitor_id'] = hm_db.id
+                    members = default_pool.pop('members', [])
+                    pool_db = self.create_pool(context, default_pool)
+                    listener['default_pool_id'] = pool_db.id
+                    for member in members:
+                        member['pool_id'] = pool_db.id
+                        self.create_pool_member(context, member, pool_db.id)
+                l7policies = listener.pop('l7policies', None)
+                listener_db = self.create_listener(context, listener)
+                if (l7policies and l7policies !=
+                        n_const.ATTR_NOT_SPECIFIED):
+                    for l7policy in l7policies:
+                        l7policy['listener_id'] = listener_db.id
+                        redirect_pool = l7policy.pop('redirect_pool', None)
+                        l7rules = l7policy.pop('rules', [])
+                        if (redirect_pool and redirect_pool !=
+                                n_const.ATTR_NOT_SPECIFIED):
+                            redirect_pool['loadbalancer_id'] = lb_db.id
+                            rhm = redirect_pool.pop('healthmonitor', None)
+                            rmembers = redirect_pool.pop('members', [])
+                            if rhm and rhm != n_const.ATTR_NOT_SPECIFIED:
+                                rhm_db = self.create_healthmonitor(context,
+                                                                   rhm)
+                                redirect_pool['healthmonitor_id'] = rhm_db.id
+                            rpool_db = self.create_pool(context, redirect_pool)
+                            l7policy['redirect_pool_id'] = rpool_db.id
+                            for rmember in rmembers:
+                                rmember['pool_id'] = rpool_db.id
+                                self.create_pool_member(context, rmember,
+                                                        rpool_db.id)
+                        l7policy_db = self.create_l7policy(context, l7policy)
+                        l7policies_ids.append(l7policy_db.id)
+                        if (l7rules and l7rules !=
+                                n_const.ATTR_NOT_SPECIFIED):
+                            for l7rule in l7rules:
+                                self.create_l7policy_rule(
+                                    context, l7rule, l7policy_db.id)
+        # SQL Alchemy cache issue where l7rules won't show up as intended.
+        for l7policy_id in l7policies_ids:
+            l7policy_db = self._get_resource(context, models.L7Policy,
+                                           l7policy_id)
+            context.session.expire(l7policy_db)
+        return self.get_loadbalancer(context, lb_db.id)
+
+    def create_loadbalancer(self, context, loadbalancer, allocate_vip=True):
+        self._load_id(context, loadbalancer)
+        vip_network_id = loadbalancer.pop('vip_network_id', None)
+        vip_subnet_id = loadbalancer.pop('vip_subnet_id', None)
+        vip_address = loadbalancer.pop('vip_address')
+        if vip_subnet_id and vip_subnet_id != n_const.ATTR_NOT_SPECIFIED:
+            loadbalancer['vip_subnet_id'] = vip_subnet_id
+        loadbalancer['provisioning_status'] = constants.PENDING_CREATE
+        loadbalancer['operating_status'] = lb_const.OFFLINE
+        lb_db = models.LoadBalancer(**loadbalancer)
 
         # create port outside of lb create transaction since it can sometimes
         # cause lock wait timeouts
         if allocate_vip:
             LOG.debug("Plugin will allocate the vip as a neutron port.")
-            try:
-                self._create_port_for_load_balancer(context, lb_db,
-                                                    vip_address)
-            except Exception:
-                with excutils.save_and_reraise_exception():
-                    context.session.delete(lb_db)
-                    context.session.flush()
+            self._create_port_for_load_balancer(context, lb_db,
+                                                vip_address, vip_network_id)
+
+        with context.session.begin(subtransactions=True):
+            context.session.add(lb_db)
+            context.session.flush()
+            lb_db.stats = self._create_loadbalancer_stats(
+                context, lb_db.id)
+            context.session.add(lb_db)
+            context.session.flush()
         return data_models.LoadBalancer.from_sqlalchemy_model(lb_db)
 
     def update_loadbalancer(self, context, id, loadbalancer):
@@ -241,14 +351,18 @@ class LoadBalancerPluginDbv2(base_db.CommonDbMixin,
             lb_db = self._get_resource(context, models.LoadBalancer, id)
             context.session.delete(lb_db)
         if delete_vip_port and lb_db.vip_port:
-            self._core_plugin.delete_port(context, lb_db.vip_port_id)
+            self._delete_vip_port(context, lb_db.vip_port_id)
+
+    @db_api.retry_db_errors
+    def _delete_vip_port(self, context, vip_port_id):
+        self._core_plugin.delete_port(context, vip_port_id)
 
     def prevent_lbaasv2_port_deletion(self, context, port_id):
         try:
             port_db = self._core_plugin._get_port(context, port_id)
         except n_exc.PortNotFound:
             return
-        if port_db['device_owner'] == n_constants.DEVICE_OWNER_LOADBALANCERV2:
+        if port_db['device_owner'] == n_const.DEVICE_OWNER_LOADBALANCERV2:
             filters = {'vip_port_id': [port_id]}
             if len(self.get_loadbalancers(context, filters=filters)) > 0:
                 reason = _('has device owner %s') % port_db['device_owner']
@@ -385,14 +499,17 @@ class LoadBalancerPluginDbv2(base_db.CommonDbMixin,
                 # Check for unspecified loadbalancer_id and listener_id and
                 # set to None
                 for id in ['loadbalancer_id', 'default_pool_id']:
-                    if listener.get(id) == attributes.ATTR_NOT_SPECIFIED:
+                    if listener.get(id) == n_const.ATTR_NOT_SPECIFIED:
                         listener[id] = None
 
                 self._validate_listener_data(context, listener)
                 sni_container_ids = []
                 if 'sni_container_ids' in listener:
                     sni_container_ids = listener.pop('sni_container_ids')
-                listener_db_entry = models.Listener(**listener)
+                try:
+                    listener_db_entry = models.Listener(**listener)
+                except Exception as exc:
+                    raise exc
                 for container_id in sni_container_ids:
                     sni = models.SNI(listener_id=listener_db_entry.id,
                                      tls_container_id=container_id)
@@ -486,7 +603,7 @@ class LoadBalancerPluginDbv2(base_db.CommonDbMixin,
             pool['provisioning_status'] = constants.PENDING_CREATE
             pool['operating_status'] = lb_const.OFFLINE
 
-            session_info = pool.pop('session_persistence')
+            session_info = pool.pop('session_persistence', None)
             pool_db = models.PoolV2(**pool)
 
             if session_info:
@@ -495,6 +612,7 @@ class LoadBalancerPluginDbv2(base_db.CommonDbMixin,
                 pool_db.session_persistence = s_p
 
             context.session.add(pool_db)
+        context.session.refresh(pool_db.loadbalancer)
         return data_models.Pool.from_sqlalchemy_model(pool_db)
 
     def update_pool(self, context, id, pool):
@@ -668,8 +786,11 @@ class LoadBalancerPluginDbv2(base_db.CommonDbMixin,
             loadbalancer.stats)
 
     def create_l7policy(self, context, l7policy):
-        if l7policy['redirect_pool_id'] == attributes.ATTR_NOT_SPECIFIED:
+        if (l7policy.get('redirect_pool_id') and
+                l7policy['redirect_pool_id'] == n_const.ATTR_NOT_SPECIFIED):
             l7policy['redirect_pool_id'] = None
+        if not l7policy.get('position'):
+            l7policy['position'] = 2147483647
         self._validate_l7policy_data(context, l7policy)
 
         with context.session.begin(subtransactions=True):
@@ -677,9 +798,6 @@ class LoadBalancerPluginDbv2(base_db.CommonDbMixin,
             listener_db = self._get_resource(
                 context, models.Listener, listener_id)
 
-            if not listener_db:
-                raise loadbalancerv2.EntityNotFound(
-                    name=models.Listener.NAME, id=listener_id)
             self._load_id(context, l7policy)
 
             l7policy['provisioning_status'] = constants.PENDING_CREATE
@@ -809,7 +927,6 @@ def _prevent_lbaasv2_port_delete_callback(resource, event, trigger, **kwargs):
     context = kwargs['context']
     port_id = kwargs['port_id']
     port_check = kwargs['port_check']
-    lbaasv2plugin = manager.NeutronManager.get_service_plugins().get(
-                         constants.LOADBALANCERV2)
+    lbaasv2plugin = directory.get_plugin(constants.LOADBALANCERV2)
     if lbaasv2plugin and port_check:
         lbaasv2plugin.db.prevent_lbaasv2_port_deletion(context, port_id)
