@@ -12,8 +12,8 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-from neutron.extensions import portbindings
 from neutron.plugins.common import constants
+from neutron_lib.api.definitions import portbindings
 from neutron_lib import exceptions as n_exc
 from oslo_log import log as logging
 import oslo_messaging as messaging
