@@ -17,9 +17,9 @@ import netaddr
 import threading
 import time
 
-from neutron.api.v2 import attributes
 from neutron import context
 from neutron.plugins.common import constants
+from neutron_lib import constants as n_constants
 from oslo_config import cfg
 from oslo_log import helpers as log_helpers
 from oslo_log import log as logging
@@ -79,7 +79,7 @@ class RadwareLBaaSV2Driver(base_v2_driver.RadwareLBaaSBaseV2Driver):
     # Assumptions:
     # 1) We have only one worflow that takes care of l2-l4 and service creation
     # 2) The workflow template exists on the vDirect server
-    # 3) The workflow expose one operaion named 'update' (plus ctor and dtor)
+    # 3) The workflow expose one operation named 'update' (plus ctor and dtor)
     # 4) The 'update' operation gets the loadbalancer object graph as input
     # 5) The object graph is enehanced by our code before it is sent to the
     #    workflow
@@ -213,7 +213,8 @@ class RadwareLBaaSV2Driver(base_v2_driver.RadwareLBaaSBaseV2Driver):
         _rest_wrapper(self.rest_client.call(
             'POST', tmpl_resource,
             {'parameters': dict(self.workflow_params,
-                                service_params=service)},
+                                service_params=service),
+            'tenants': [lb.tenant_id]},
             TEMPLATE_HEADER))
 
     @log_helpers.log_method_call
@@ -297,7 +298,7 @@ class RadwareLBaaSV2Driver(base_v2_driver.RadwareLBaaSBaseV2Driver):
 
     def remove_workflow(self, ctx, manager, lb):
         wf_name = self._get_wf_name(lb)
-        LOG.debug('Remove the workflow %s' % wf_name)
+        LOG.debug('Remove the workflow %s', wf_name)
         resource = '/api/workflow/%s' % (wf_name)
         rest_return = self.rest_client.call('DELETE', resource, None, None)
         response = _rest_wrapper(rest_return, [204, 202, 404])
@@ -493,7 +494,7 @@ class RadwareLBaaSV2Driver(base_v2_driver.RadwareLBaaSBaseV2Driver):
             'tenant_id': lb.tenant_id,
             'name': proxy_port_name,
             'network_id': proxy_port_subnet['network_id'],
-            'mac_address': attributes.ATTR_NOT_SPECIFIED,
+            'mac_address': n_constants.ATTR_NOT_SPECIFIED,
             'admin_state_up': False,
             'device_id': '',
             'device_owner': 'neutron:' + constants.LOADBALANCERV2,
@@ -586,7 +587,7 @@ class OperationCompletionHandler(threading.Thread):
                                        oper.operation_url,
                                        None,
                                        None)
-        LOG.debug('Operation completion requested %(uri) and got: %(result)',
+        LOG.debug('Operation completion requested %(uri)s and got: %(result)s',
                   {'uri': oper.operation_url, 'result': result})
         completed = result[rest.RESP_DATA]['complete']
         reason = result[rest.RESP_REASON],
