@@ -16,18 +16,23 @@
 
 import abc
 
+from neutron_lib.api import converters
+from neutron_lib.api import extensions as api_extensions
+from neutron_lib.api import validators
+from neutron_lib import constants as n_constants
+from neutron_lib.db import constants as db_const
+from neutron_lib import exceptions as nexception
+from neutron_lib.plugins import directory
 from oslo_config import cfg
 from oslo_log import log as logging
 import six
 
 from neutron.api import extensions
-from neutron.api.v2 import attributes as attr
 from neutron.api.v2 import base
 from neutron.api.v2 import resource_helper
-from neutron import manager
 from neutron.plugins.common import constants
+from neutron.quota import resource_registry
 from neutron.services import service_base
-from neutron_lib import exceptions as nexception
 
 from neutron_lbaas._i18n import _
 from neutron_lbaas.services.loadbalancer import constants as lb_const
@@ -100,7 +105,7 @@ class MemberAddressTypeSubnetTypeMismatch(nexception.NeutronException):
 
 
 class DriverError(nexception.NeutronException):
-    message = _("An error happened in the driver")
+    message = _("Driver error: %(msg)s")
 
 
 class SessionPersistenceConfigurationInvalid(nexception.BadRequest):
@@ -139,8 +144,7 @@ def _validate_connection_limit(data, min_value=lb_const.MIN_CONNECT_VALUE):
         LOG.debug(msg)
         return msg
 
-attr.validators['type:connection_limit'] = _validate_connection_limit
-
+validators.validators['type:connection_limit'] = _validate_connection_limit
 
 RESOURCE_ATTRIBUTE_MAP = {
     'loadbalancers': {
@@ -149,34 +153,35 @@ RESOURCE_ATTRIBUTE_MAP = {
                'is_visible': True,
                'primary_key': True},
         'name': {'allow_post': True, 'allow_put': True,
-                 'validate': {'type:string': attr.NAME_MAX_LEN},
+                 'validate': {'type:string': db_const.NAME_FIELD_SIZE},
                  'default': '',
                  'is_visible': True},
         'tenant_id': {'allow_post': True, 'allow_put': False,
                       'validate': {'type:not_empty_string':
-                                   attr.TENANT_ID_MAX_LEN},
+                                   db_const.PROJECT_ID_FIELD_SIZE},
                       'required_by_policy': True,
                       'is_visible': True},
         'description': {'allow_post': True, 'allow_put': True,
-                        'validate': {'type:string': attr.DESCRIPTION_MAX_LEN},
+                        'validate': {
+                            'type:string': db_const.DESCRIPTION_FIELD_SIZE},
                         'is_visible': True, 'default': ''},
         'vip_subnet_id': {'allow_post': True, 'allow_put': False,
                           'validate': {'type:uuid': None},
                           'is_visible': True},
         'vip_address': {'allow_post': True, 'allow_put': False,
-                        'default': attr.ATTR_NOT_SPECIFIED,
+                        'default': n_constants.ATTR_NOT_SPECIFIED,
                         'validate': {'type:ip_address_or_none': None},
                         'is_visible': True},
         'vip_port_id': {'allow_post': False, 'allow_put': False,
                         'is_visible': True},
         'provider': {'allow_post': True, 'allow_put': False,
-                     'validate': {'type:string': None},
-                     'is_visible': True, 'default': attr.ATTR_NOT_SPECIFIED},
+                     'validate': {'type:string': None}, 'is_visible': True,
+                     'default': n_constants.ATTR_NOT_SPECIFIED},
         'listeners': {'allow_post': False, 'allow_put': False,
                       'is_visible': True},
         'admin_state_up': {'allow_post': True, 'allow_put': True,
                            'default': True,
-                           'convert_to': attr.convert_to_boolean,
+                           'convert_to': converters.convert_to_boolean,
                            'is_visible': True},
         'provisioning_status': {'allow_post': False, 'allow_put': False,
                                 'is_visible': True},
@@ -184,8 +189,8 @@ RESOURCE_ATTRIBUTE_MAP = {
                              'is_visible': True},
         'flavor_id': {'allow_post': True, 'allow_put': False,
                       'is_visible': True,
-                      'validate': {'type:string': attr.NAME_MAX_LEN},
-                      'default': attr.ATTR_NOT_SPECIFIED}
+                      'validate': {'type:string': db_const.NAME_FIELD_SIZE},
+                      'default': n_constants.ATTR_NOT_SPECIFIED}
     },
     'listeners': {
         'id': {'allow_post': False, 'allow_put': False,
@@ -194,15 +199,16 @@ RESOURCE_ATTRIBUTE_MAP = {
                'primary_key': True},
         'tenant_id': {'allow_post': True, 'allow_put': False,
                       'validate': {'type:not_empty_string':
-                                   attr.TENANT_ID_MAX_LEN},
+                                   db_const.PROJECT_ID_FIELD_SIZE},
                       'required_by_policy': True,
                       'is_visible': True},
         'name': {'allow_post': True, 'allow_put': True,
-                 'validate': {'type:string': attr.NAME_MAX_LEN},
+                 'validate': {'type:string': db_const.NAME_FIELD_SIZE},
                  'default': '',
                  'is_visible': True},
         'description': {'allow_post': True, 'allow_put': True,
-                        'validate': {'type:string': attr.DESCRIPTION_MAX_LEN},
+                        'validate': {
+                            'type:string': db_const.DESCRIPTION_FIELD_SIZE},
                         'is_visible': True, 'default': ''},
         'loadbalancer_id': {'allow_post': True, 'allow_put': False,
                             'validate': {'type:uuid': None},
@@ -219,25 +225,25 @@ RESOURCE_ATTRIBUTE_MAP = {
                                       'is_visible': True},
         'sni_container_refs': {'allow_post': True, 'allow_put': True,
                                'default': None,
-                               'convert_to': attr.convert_to_list,
+                               'convert_to': converters.convert_to_list,
                                'is_visible': True},
         'connection_limit': {'allow_post': True, 'allow_put': True,
                              'validate': {'type:connection_limit':
                                           lb_const.MIN_CONNECT_VALUE},
                              'default': lb_const.MIN_CONNECT_VALUE,
-                             'convert_to': attr.convert_to_int,
+                             'convert_to': converters.convert_to_int,
                              'is_visible': True},
         'protocol': {'allow_post': True, 'allow_put': False,
                      'validate': {'type:values':
                                   lb_const.LISTENER_SUPPORTED_PROTOCOLS},
                      'is_visible': True},
         'protocol_port': {'allow_post': True, 'allow_put': False,
-                          'validate': {'type:range': [0, 65535]},
-                          'convert_to': attr.convert_to_int,
+                          'validate': {'type:range': [1, 65535]},
+                          'convert_to': converters.convert_to_int,
                           'is_visible': True},
         'admin_state_up': {'allow_post': True, 'allow_put': True,
                            'default': True,
-                           'convert_to': attr.convert_to_boolean,
+                           'convert_to': converters.convert_to_boolean,
                            'is_visible': True}
     },
     'pools': {
@@ -247,14 +253,15 @@ RESOURCE_ATTRIBUTE_MAP = {
                'primary_key': True},
         'tenant_id': {'allow_post': True, 'allow_put': False,
                       'validate': {'type:not_empty_string':
-                                   attr.TENANT_ID_MAX_LEN},
+                                   db_const.PROJECT_ID_FIELD_SIZE},
                       'required_by_policy': True,
                       'is_visible': True},
         'name': {'allow_post': True, 'allow_put': True,
-                 'validate': {'type:string': attr.NAME_MAX_LEN},
+                 'validate': {'type:string': db_const.NAME_FIELD_SIZE},
                  'is_visible': True, 'default': ''},
         'description': {'allow_post': True, 'allow_put': True,
-                        'validate': {'type:string': attr.DESCRIPTION_MAX_LEN},
+                        'validate': {
+                            'type:string': db_const.DESCRIPTION_FIELD_SIZE},
                         'is_visible': True, 'default': ''},
         'listener_id': {'allow_post': True, 'allow_put': False,
                         'validate': {'type:uuid': None},
@@ -274,7 +281,7 @@ RESOURCE_ATTRIBUTE_MAP = {
                          'is_visible': True},
         'session_persistence': {
             'allow_post': True, 'allow_put': True,
-            'convert_to': attr.convert_none_to_empty_dict,
+            'convert_to': converters.convert_none_to_empty_dict,
             'default': {},
             'validate': {
                 'type:dict_or_empty': {
@@ -288,7 +295,7 @@ RESOURCE_ATTRIBUTE_MAP = {
                     'is_visible': True},
         'admin_state_up': {'allow_post': True, 'allow_put': True,
                            'default': True,
-                           'convert_to': attr.convert_to_boolean,
+                           'convert_to': converters.convert_to_boolean,
                            'is_visible': True}
     },
     'healthmonitors': {
@@ -298,7 +305,7 @@ RESOURCE_ATTRIBUTE_MAP = {
                'primary_key': True},
         'tenant_id': {'allow_post': True, 'allow_put': False,
                       'validate': {'type:not_empty_string':
-                                   attr.TENANT_ID_MAX_LEN},
+                                   db_const.PROJECT_ID_FIELD_SIZE},
                       'required_by_policy': True,
                       'is_visible': True},
         'pool_id': {'allow_post': True, 'allow_put': False,
@@ -312,15 +319,15 @@ RESOURCE_ATTRIBUTE_MAP = {
                  'is_visible': True},
         'delay': {'allow_post': True, 'allow_put': True,
                   'validate': {'type:non_negative': None},
-                  'convert_to': attr.convert_to_int,
+                  'convert_to': converters.convert_to_int,
                   'is_visible': True},
         'timeout': {'allow_post': True, 'allow_put': True,
                     'validate': {'type:non_negative': None},
-                    'convert_to': attr.convert_to_int,
+                    'convert_to': converters.convert_to_int,
                     'is_visible': True},
         'max_retries': {'allow_post': True, 'allow_put': True,
                         'validate': {'type:range': [1, 10]},
-                        'convert_to': attr.convert_to_int,
+                        'convert_to': converters.convert_to_int,
                         'is_visible': True},
         'http_method': {'allow_post': True, 'allow_put': True,
                         'validate': {'type:values':
@@ -343,10 +350,10 @@ RESOURCE_ATTRIBUTE_MAP = {
         },
         'admin_state_up': {'allow_post': True, 'allow_put': True,
                            'default': True,
-                           'convert_to': attr.convert_to_boolean,
+                           'convert_to': converters.convert_to_boolean,
                            'is_visible': True},
         'name': {'allow_post': True, 'allow_put': True,
-                 'validate': {'type:string': attr.NAME_MAX_LEN},
+                 'validate': {'type:string': db_const.NAME_FIELD_SIZE},
                  'default': '',
                  'is_visible': True}
     }
@@ -363,30 +370,30 @@ SUB_RESOURCE_ATTRIBUTE_MAP = {
                    'primary_key': True},
             'tenant_id': {'allow_post': True, 'allow_put': False,
                           'validate': {'type:not_empty_string':
-                                       attr.TENANT_ID_MAX_LEN},
+                                       db_const.PROJECT_ID_FIELD_SIZE},
                           'required_by_policy': True,
                           'is_visible': True},
             'address': {'allow_post': True, 'allow_put': False,
                         'validate': {'type:ip_address': None},
                         'is_visible': True},
             'protocol_port': {'allow_post': True, 'allow_put': False,
-                              'validate': {'type:range': [0, 65535]},
-                              'convert_to': attr.convert_to_int,
+                              'validate': {'type:range': [1, 65535]},
+                              'convert_to': converters.convert_to_int,
                               'is_visible': True},
             'weight': {'allow_post': True, 'allow_put': True,
                        'default': 1,
-                       'validate': {'type:range': [0, 256]},
-                       'convert_to': attr.convert_to_int,
+                       'validate': {'type:range': [0, 256],
+                                    'type:integer': None},
                        'is_visible': True},
             'admin_state_up': {'allow_post': True, 'allow_put': True,
                                'default': True,
-                               'convert_to': attr.convert_to_boolean,
+                               'convert_to': converters.convert_to_boolean,
                                'is_visible': True},
             'subnet_id': {'allow_post': True, 'allow_put': False,
                           'validate': {'type:uuid': None},
                           'is_visible': True},
             'name': {'allow_post': True, 'allow_put': True,
-                     'validate': {'type:string': attr.NAME_MAX_LEN},
+                     'validate': {'type:string': db_const.NAME_FIELD_SIZE},
                      'default': '',
                      'is_visible': True},
         }
@@ -419,7 +426,7 @@ lbaasv2_quota_opts = [
 cfg.CONF.register_opts(lbaasv2_quota_opts, 'QUOTAS')
 
 
-class Loadbalancerv2(extensions.ExtensionDescriptor):
+class Loadbalancerv2(api_extensions.ExtensionDescriptor):
 
     @classmethod
     def get_name(cls):
@@ -447,22 +454,20 @@ class Loadbalancerv2(extensions.ExtensionDescriptor):
             {}, RESOURCE_ATTRIBUTE_MAP)
         action_map = {'loadbalancer': {'stats': 'GET', 'statuses': 'GET'}}
         plural_mappings['members'] = 'member'
+        resource_registry.register_resource_by_name('member', 'members')
         plural_mappings['sni_container_refs'] = 'sni_container_ref'
         plural_mappings['sni_container_ids'] = 'sni_container_id'
-        attr.PLURALS.update(plural_mappings)
         resources = resource_helper.build_resource_info(
             plural_mappings,
             RESOURCE_ATTRIBUTE_MAP,
             constants.LOADBALANCERV2,
             action_map=action_map,
             register_quota=True)
-        plugin = manager.NeutronManager.get_service_plugins()[
-            constants.LOADBALANCERV2]
+        plugin = directory.get_plugin(constants.LOADBALANCERV2)
         for collection_name in SUB_RESOURCE_ATTRIBUTE_MAP:
             # Special handling needed for sub-resources with 'y' ending
             # (e.g. proxies -> proxy)
             resource_name = collection_name[:-1]
-
             parent = SUB_RESOURCE_ATTRIBUTE_MAP[collection_name].get('parent')
             params = SUB_RESOURCE_ATTRIBUTE_MAP[collection_name].get(
                 'parameters')
@@ -668,3 +673,7 @@ class LoadBalancerPluginBaseV2(service_base.ServicePluginBase):
     @abc.abstractmethod
     def delete_l7policy_rule(self, context, id, l7policy_id):
         pass
+
+    @abc.abstractmethod
+    def create_graph(self, context, graph):
+        pass
