@@ -14,15 +14,17 @@
 #    under the License.
 
 import copy
-import six
+
+from neutron_lib.plugins import directory
 
 from neutron.api.v2 import attributes as attrs
 from neutron.api.v2 import base as napi_base
 from neutron import context as ncontext
+from neutron.db import agentschedulers_db
 from neutron.db import servicetype_db as st_db
 from neutron.extensions import flavors
-from neutron import manager
 from neutron.plugins.common import constants
+from neutron import service
 from neutron.services.flavors import flavors_plugin
 from neutron.services import provider_configuration as pconf
 from neutron.services import service_base
@@ -54,7 +56,8 @@ def add_provider_configuration(type_manager, service_type):
         pconf.ProviderConfiguration('neutron_lbaas'))
 
 
-class LoadBalancerPluginv2(loadbalancerv2.LoadBalancerPluginBaseV2):
+class LoadBalancerPluginv2(loadbalancerv2.LoadBalancerPluginBaseV2,
+                           agentschedulers_db.AgentSchedulerDbMixin):
     """Implementation of the Neutron Loadbalancer Service Plugin.
 
     This class manages the workflow of LBaaS request/response.
@@ -81,8 +84,17 @@ class LoadBalancerPluginv2(loadbalancerv2.LoadBalancerPluginBaseV2):
         add_provider_configuration(
             self.service_type_manager, constants.LOADBALANCERV2)
         self._load_drivers()
+        self.start_periodic_jobs()
         self.start_rpc_listeners()
         self.db.subscribe()
+        rpc_worker = service.RpcWorker([self], worker_process_count=0)
+        self.add_worker(rpc_worker)
+
+    def start_periodic_jobs(self):
+        for driver_name, driver_class in self.drivers.items():
+            if hasattr(driver_class, 'get_periodic_jobs'):
+                for job in self.drivers[driver_name].get_periodic_jobs():
+                    self.add_agent_status_check_worker(job)
 
     def start_rpc_listeners(self):
         listeners = []
@@ -212,8 +224,7 @@ class LoadBalancerPluginv2(loadbalancerv2.LoadBalancerPluginBaseV2):
             loadbalancer['provider'] != n_constants.ATTR_NOT_SPECIFIED):
             raise loadbalancerv2.ProviderFlavorConflict()
 
-        plugin = manager.NeutronManager.get_service_plugins().get(
-            constants.FLAVORS)
+        plugin = directory.get_plugin(constants.FLAVORS)
         if not plugin:
             raise loadbalancerv2.FlavorsPluginNotLoaded()
 
@@ -665,10 +676,9 @@ class LoadBalancerPluginv2(loadbalancerv2.LoadBalancerPluginBaseV2):
         # blank out the listeners at this point.
         del pool['listener_id']
         pool['listeners'] = []
-        db_pool = self.db.create_pool(context, pool)
         self.db.test_and_set_status(context, models.LoadBalancer,
-                                    db_pool.loadbalancer_id,
-                                    constants.PENDING_UPDATE)
+                                    lb_id, constants.PENDING_UPDATE)
+        db_pool = self.db.create_pool(context, pool)
         for db_l in db_listeners:
             try:
                 self.db.update_listener(context, db_l.id,
@@ -1034,7 +1044,7 @@ class LoadBalancerPluginv2(loadbalancerv2.LoadBalancerPluginBaseV2):
             status["provisioning_status"] = obj.provisioning_status
         if "operating_status" not in exclude:
             status["operating_status"] = obj.operating_status
-        for key, value in six.iteritems(kw):
+        for key, value in kw.items():
             status[key] = value
         try:
             status['name'] = getattr(obj, 'name')
@@ -1211,7 +1221,7 @@ class LoadBalancerPluginv2(loadbalancerv2.LoadBalancerPluginBaseV2):
             if healthmonitor:
                 if not healthmonitor.admin_state_up:
                     dhm = self._disable_entity_and_children(healthmonitor)
-                    hm_status = dhm
+                    pool_status["healthmonitor"] = dhm
                 else:
                     hm_status = self._default_status(healthmonitor,
                                 exclude=[OS], type=healthmonitor.type)
