@@ -3,16 +3,7 @@
 set -ex
 
 GATE_DEST=$BASE/new
-
-_DEVSTACK_LOCAL_CONFIG_TAIL=
-
-# Inject config from hook
-function load_conf_hook {
-    local hook="$1"
-    local GATE_HOOKS=$GATE_DEST/neutron-lbaas/neutron_lbaas/tests/contrib/hooks
-
-    _DEVSTACK_LOCAL_CONFIG_TAIL+=$'\n'"$(cat $GATE_HOOKS/$hook)"
-}
+DEVSTACK_PATH=$GATE_DEST/devstack
 
 export DEVSTACK_LOCAL_CONFIG+="
 enable_plugin neutron-lbaas https://git.openstack.org/openstack/neutron-lbaas
@@ -31,15 +22,39 @@ function _setup_octavia {
     export DEVSTACK_LOCAL_CONFIG+="
         enable_plugin octavia https://git.openstack.org/openstack/octavia
         "
+    # Use infra's cached version of the file
+    if [ -f /opt/stack/new/devstack/files/get-pip.py ]; then
+            export DEVSTACK_LOCAL_CONFIG+="
+        DIB_REPOLOCATION_pip_and_virtualenv=file:///opt/stack/new/devstack/files/get-pip.py
+        "
+    fi
+
     if [ "$testenv" != "apiv1" ]; then
         ENABLED_SERVICES+="octavia,o-cw,o-hk,o-hm,o-api,"
     fi
     if [ "$testenv" = "apiv2" ]; then
-        load_conf_hook apiv2
+       cat > "$DEVSTACK_PATH/local.conf" <<EOF
+[[post-config|/etc/octavia/octavia.conf]]
+[DEFAULT]
+debug = True
+
+[controller_worker]
+amphora_driver = amphora_noop_driver
+compute_driver = compute_noop_driver
+network_driver = network_noop_driver
+
+EOF
+
     fi
 
     if [ "$testenv" = "scenario" ]; then
-        load_conf_hook scenario
+       cat > "$DEVSTACK_PATH/local.conf" <<EOF
+[[post-config|/etc/octavia/octavia.conf]]
+[DEFAULT]
+debug = True
+
+EOF
+
     fi
 }
 
@@ -49,6 +64,7 @@ case "$testtype" in
     "dsvm-functional")
         PROJECT_NAME=neutron-lbaas
         NEUTRON_LBAAS_PATH=$GATE_DEST/$PROJECT_NAME
+        DEVSTACK_PATH=$GATE_DEST/devstack
         IS_GATE=True
         USE_CONSTRAINT_ENV=False
         export LOG_COLOR=False
@@ -61,9 +77,6 @@ case "$testtype" in
         ;;
 
     "tempest")
-        # Make sure lbaasv2 is listed as enabled for tempest
-        load_conf_hook api_extensions
-
         # These are not needed with either v1 or v2
         ENABLED_SERVICES+="-c-api,-c-bak,-c-sch,-c-vol,-cinder,"
         ENABLED_SERVICES+="-s-account,-s-container,-s-object,-s-proxy,"
@@ -98,7 +111,6 @@ case "$testtype" in
         fi
 
         export ENABLED_SERVICES
-        export DEVSTACK_LOCAL_CONFIG+=$'\n'"$_DEVSTACK_LOCAL_CONFIG_TAIL"
         "$GATE_DEST"/devstack-gate/devstack-vm-gate.sh
         ;;
 
