@@ -13,8 +13,7 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import contextlib
-
+import collections
 import mock
 from neutron.plugins.common import constants
 
@@ -45,7 +44,8 @@ class TestManager(base.BaseTestCase):
         self.log = mock.patch.object(manager, 'LOG').start()
         self.driver_mock = mock.Mock()
         self.mgr.device_drivers = {'devdriver': self.driver_mock}
-        self.mgr.instance_mapping = {'1': 'devdriver', '2': 'devdriver'}
+        instance_mapping = [('1', 'devdriver'), ('2', 'devdriver')]
+        self.mgr.instance_mapping = collections.OrderedDict(instance_mapping)
         self.mgr.needs_resync = False
         self.update_statuses_patcher = mock.patch.object(
             self.mgr, '_update_statuses')
@@ -85,11 +85,9 @@ class TestManager(base.BaseTestCase):
         self.assertTrue(self.log.exception.called)
 
     def _sync_state_helper(self, ready, reloaded, destroyed):
-        with contextlib.nested(
-            mock.patch.object(self.mgr, '_reload_loadbalancer'),
-            mock.patch.object(self.mgr, '_destroy_loadbalancer')
-        ) as (reload, destroy):
-
+        with mock.patch.object(self.mgr, '_reload_loadbalancer') as reload, \
+                mock.patch.object(self.mgr, '_destroy_loadbalancer') as \
+                destroy:
             self.rpc_mock.get_ready_devices.return_value = ready
 
             self.mgr.sync_state()
@@ -206,8 +204,7 @@ class TestManager(base.BaseTestCase):
 
     def test_remove_orphans(self):
         self.mgr.remove_orphans()
-        orphans = {'1': "Fake", '2': "Fake"}
-        self.driver_mock.remove_orphans.assert_called_once_with(orphans.keys())
+        self.driver_mock.remove_orphans.assert_called_once_with(['1', '2'])
 
     def test_agent_disabled(self):
         payload = {'admin_state_up': False}
