@@ -14,6 +14,7 @@
 
 import mock
 from oslo_config import cfg
+from oslo_messaging.rpc import dispatcher
 
 from neutron_lbaas.common import exceptions
 from neutron_lbaas.db.loadbalancer import models
@@ -83,9 +84,10 @@ class TestOctaviaMessagingConsumer(test_octavia_driver.BaseOctaviaDriverTest):
                               group='oslo_messaging')
         consumer = octavia_messaging_consumer.OctaviaConsumer(self.driver)
         consumer.start()
+        access_policy = dispatcher.DefaultRPCAccessPolicy
         mock_get_rpc_server.assert_called_once_with(
             consumer.transport, consumer.target, consumer.endpoints,
-            executor='eventlet'
+            executor='eventlet', access_policy=access_policy
         )
         mock_server.start.assert_called_once_with()
 
@@ -136,21 +138,6 @@ class TestOctaviaMessagingConsumer(test_octavia_driver.BaseOctaviaDriverTest):
         self.assertRaises(exceptions.ModelMapException,
                           self.consumer.endpoints[0].update_info, {}, cnt)
 
-    def test_update_loadbalancer_stats(self):
-        self.set_db_mocks()
-        stats = {
-            'bytes_in': 1,
-            'bytes_out': 2,
-            'active_connections': 3,
-            'total_connections': 4,
-            'request_errors': 5,
-        }
-        cnt = InfoContainer(constants.LOADBALANCER_STATS_EVENT, 'lb_id',
-                            stats).to_dict()
-        self.consumer.endpoints[0].update_info({}, cnt)
-        self.driver.plugin.db.update_loadbalancer_stats.assert_called_with(
-            mock.ANY, 'lb_id', stats)
-
     def test_updatedb_ignores_listener_stats(self):
         self.set_db_mocks()
         cnt = InfoContainer('listener_stats', 'id', self.payload).to_dict()
