@@ -18,19 +18,20 @@ import copy
 from datetime import datetime
 
 import mock
+from neutron_lib.plugins import directory
+
 from neutron.api import extensions
 from neutron.api.v2 import attributes
 from neutron import context
 from neutron.db import agents_db
 from neutron.extensions import agent
-from neutron import manager
 from neutron.plugins.common import constants as plugin_const
 from neutron.tests.common import helpers
 from neutron.tests.unit.api import test_extensions
 from neutron.tests.unit.db import test_agentschedulers_db
 import neutron.tests.unit.extensions
 from neutron.tests.unit.extensions import test_agent
-import six
+from neutron_lib import constants as n_constants
 from webob import exc
 
 from neutron_lbaas.drivers.haproxy import plugin_driver
@@ -96,7 +97,7 @@ class LBaaSAgentSchedulerTestCase(test_agent.AgentDBTestMixIn,
     def setUp(self):
         # Save the global RESOURCE_ATTRIBUTE_MAP
         self.saved_attr_map = {}
-        for res, attrs in six.iteritems(attributes.RESOURCE_ATTRIBUTE_MAP):
+        for res, attrs in attributes.RESOURCE_ATTRIBUTE_MAP.items():
             self.saved_attr_map[res] = attrs.copy()
         service_plugins = {
             'lb_plugin_name': test_db_loadbalancerv2.DB_LB_PLUGIN_CLASS}
@@ -117,9 +118,8 @@ class LBaaSAgentSchedulerTestCase(test_agent.AgentDBTestMixIn,
         # the global attribute map
         attributes.RESOURCE_ATTRIBUTE_MAP.update(
             agent.RESOURCE_ATTRIBUTE_MAP)
-        self.lbaas_plugin = manager.NeutronManager.get_service_plugins()[
-            plugin_const.LOADBALANCERV2]
-        self.core_plugin = manager.NeutronManager.get_plugin()
+        self.lbaas_plugin = directory.get_plugin(plugin_const.LOADBALANCERV2)
+        self.core_plugin = directory.get_plugin()
         self.addCleanup(self.restore_attribute_map)
 
     def restore_attribute_map(self):
@@ -172,10 +172,11 @@ class LBaaSAgentSchedulerTestCase(test_agent.AgentDBTestMixIn,
             'loadbalancer': {
                 'vip_subnet_id': subnet['id'],
                 'provider': 'lbaas',
-                'flavor_id': attributes.ATTR_NOT_SPECIFIED,
-                'vip_address': attributes.ATTR_NOT_SPECIFIED,
+                'flavor_id': n_constants.ATTR_NOT_SPECIFIED,
+                'vip_address': n_constants.ATTR_NOT_SPECIFIED,
                 'admin_state_up': True,
-                'tenant_id': self._tenant_id}}
+                'tenant_id': self._tenant_id,
+                'listeners': []}}
         self.assertRaises(lbaas_agentschedulerv2.NoEligibleLbaasAgent,
                           self.lbaas_plugin.create_loadbalancer,
                           self.adminContext, lb)
@@ -190,7 +191,7 @@ class LBaaSAgentSchedulerTestCase(test_agent.AgentDBTestMixIn,
             ]},
             'agent_type': lb_const.AGENT_TYPE_LOADBALANCERV2}
         helpers._register_agent(lbaas_hosta)
-        is_agent_down_str = 'neutron.db.agents_db.AgentDbMixin.is_agent_down'
+        is_agent_down_str = 'neutron.agent.common.utils.is_agent_down'
         with mock.patch(is_agent_down_str) as mock_is_agent_down:
             mock_is_agent_down.return_value = False
             with self.loadbalancer() as loadbalancer:
@@ -207,10 +208,11 @@ class LBaaSAgentSchedulerTestCase(test_agent.AgentDBTestMixIn,
                 'loadbalancer': {
                     'vip_subnet_id': subnet['id'],
                     'provider': 'lbaas',
-                    'flavor_id': attributes.ATTR_NOT_SPECIFIED,
-                    'vip_address': attributes.ATTR_NOT_SPECIFIED,
+                    'flavor_id': n_constants.ATTR_NOT_SPECIFIED,
+                    'vip_address': n_constants.ATTR_NOT_SPECIFIED,
                     'admin_state_up': True,
-                    'tenant_id': self._tenant_id}}
+                    'tenant_id': self._tenant_id,
+                    'listeners': []}}
             self.assertRaises(lbaas_agentschedulerv2.NoEligibleLbaasAgent,
                               self.lbaas_plugin.create_loadbalancer,
                               self.adminContext, lb)
